@@ -35,6 +35,7 @@
   #:use-module (artanis third-party json)
   #:use-module (artanis server)
   #:use-module (artanis websocket)
+  #:use-module (artanis lpc)
   #:use-module (artanis version)
   #:use-module (web server)
   #:use-module (srfi srfi-1)
@@ -50,6 +51,7 @@
                response-error-emit
                run-after-request!
                run-before-response!
+               run-when-sigint!
                emit-response-with-file
                static-page-emitter
 
@@ -75,6 +77,7 @@
                :session
                :from-post
                :websocket
+               :lpc
 
                ;; db module
                DB-open
@@ -125,6 +128,7 @@
                route-context?
                get-header
                get-from-qstr
+               get-referer
 
                ;; csv
                make-csv-reader
@@ -141,7 +145,6 @@
                json-string->scm
                json-parser?
                json-parser-port
-               json
 
                ;; upload
                mfd-simple-dump
@@ -165,6 +168,9 @@
                store-uploaded-files
                upload-files-to
 
+               ;; websocket
+               named-pipe-subscribe
+
                ;; server
                schedule-task
 
@@ -200,6 +206,7 @@
   (default-route-init statics cache-statics? exclude)
   (init-hook)
   (init-config)
+  (init-lpc)
   (check-invalid-config)
   (sigaction SIGPIPE SIG_IGN) ; surpass SIGPIPE signal since we want to handle EPIPE by self
   (set! is-init-server-run? #t))
@@ -278,7 +285,7 @@
       (if debug
           (print-websocket-frame body)
           "<websocket-frame>"))
-     (else (throw 'artanis-err 500 ->proper-body-display "Invalid body type!" body))))
+     (else (throw 'artanis-err 500 ->proper-body-display "Invalid body type `~a'!" body))))
   (when (not is-init-server-run?)
     (error "Sorry, but you have to run (init-server) in the begining of you main program!"))
   (and host (conf-set! '(host addr) host))
@@ -288,8 +295,8 @@
   (when debug
     (display "DEBUG: ON\n")
     (init-debug-mode))
-  (when (or use-db? (get-conf 'use-db?))
-    (conf-set! 'use-db? #t)
+  (when (or use-db? (get-conf '(db enable)))
+    (conf-set! '(db enable) #t)
     (display "User wants to use Database, initializing...\n")
     (init-database-config dbd db-username db-passwd db-name db-addr db-proto)
     (init-DB)
@@ -299,7 +306,7 @@
       (display "Session with DB backend init done!\n")))
   (case (get-conf '(session backend))
     ((db)
-     (when (not (get-conf 'use-db?))
+     (when (not (get-conf '(db enable)))
        (error "Session with DB backend init failed because you didn't enable DB!")))
     (else
      (session-init)
