@@ -22,12 +22,14 @@
   #:use-module (artanis env)
   #:use-module (artanis config)
   #:use-module (artanis cookie)
+  #:use-module (artanis lpc)
   #:use-module (artanis tpl)
   #:use-module (artanis tpl sxml)
   #:use-module (artanis db)
   #:use-module (artanis route)
   #:use-module (artanis websocket)
   #:use-module (artanis server http)
+  #:use-module (artanis server server-context)
   #:use-module (srfi srfi-19)
   #:use-module (web uri)
   #:use-module (web http)
@@ -58,6 +60,42 @@
 (define (rc-conn-recycle rc body)
   (and=> (rc-conn rc) DB-close))
 
+(define (rc-lpc-recycle rc body)
+  (and=> (rc-lpc rc) lpc-instance-recycle))
+
+(define (try-to-register-websocket-pipe! req new-client)
+  (let ((name (detect-pipe-name req)))
+    (DEBUG "Register named-pipe: ~a~%" name)
+    (cond
+     ((get-named-pipe name)
+      => (lambda (named-pipe)
+           (let ((old-client (named-pipe-client named-pipe))
+                 (server (current-server)))
+             (DEBUG "Replace existing websocket ~a with ~a in named-pipe ~a~%"
+                    (client-sockport old-client) (client-sockport new-client) name)
+             ;; NOTE: If the same name was specified, we close the old one then register the
+             ;;       new one. This is because some clients/browswers have bugs to not close
+             ;;       connection when refresh the page/webapi, so we close it positively to
+             ;;       avoid further problem.
+             ;; FIXME: We should detect secure token here first.
+             ;; NOTE: Because WS connection was closed then reopened, so we MUST pass
+             ;;       peer-shutdown? as #f here, or it'll close the new WS connection.
+             (DEBUG "Closing handshake of replaced WS connection~%")
+             (closing-websocket-handshake server old-client #f)
+             (DEBUG "Closing replaced WS connection~%")
+             (http-close server old-client #t)
+             (DEBUG "Replace existing named-pipe ~a ...~%" name)
+             (named-pipe-client-set! named-pipe new-client)
+             (register-websocket-pipe! named-pipe)
+             (DEBUG "done."))))
+     (else (register-websocket-pipe! (new-named-pipe name new-client))))))
+
+(define (print-request-info rq body)
+  (let ((path (request-path rq))
+        (method (request-method rq))
+        (port (request-port rq)))
+    (format (artanis-current-output) "[Client] ~a ~a ~a~%" method path port)))
+
 (define (run-after-request-hooks rq body)
   (run-hook *after-request-hook* rq body))
 
@@ -65,15 +103,24 @@
   (run-hook *before-response-hook* rc body))
 
 (define (init-after-request-hook)
-  #t)
+  (run-after-request! print-request-info))
 
 (define (init-before-response-hook)
-  (run-before-response! rc-conn-recycle))
+  (run-before-response! rc-conn-recycle)
+  (run-before-response! rc-lpc-recycle))
+
+(define (init-after-websocket-hook)
+  (run-after-websocket-handshake! try-to-register-websocket-pipe!))
+
+(define (init-startup-hook)
+  #t)
 
 ;; NOTE: If you want to add hook during initialization time, put them here.
 (define (init-hook)
+  (init-startup-hook)
   (init-after-request-hook)
-  (init-before-response-hook))
+  (init-before-response-hook)
+  (init-after-websocket-hook))
 
 (define (handler-render handler rc)
   (define (->bytevector body)
@@ -91,6 +138,7 @@
                    (status 200)
                    (mtime (generate-modify-time (current-time)))
                    (request-status 'ok))
+      (run-before-response-hooks rc body)
       (let* ((reformed-body (->bytevector body))
              (response
               (build-response #:code status
@@ -99,7 +147,6 @@
                                           ,(gen-content-length reformed-body)
                                           ,@pre-headers
                                           ,@(generate-cookies (rc-set-cookie rc))))))
-        (run-before-response-hooks rc body)
         (let ((type (assq-ref pre-headers 'content-type)))
           (and type (artanis-log 'client status (car type) #:request (rc-req rc))))
         ;; NOTE: For inner-server, sanitize-response will handle 'HEAD method
@@ -166,8 +213,11 @@
   (response-emit
    ""
    #:status status
-   #:headers `((location . ,(build-uri scheme #:path path))
-               (content-length . 0)
+   #:headers `((location . ,(cond
+                             ((string? path) (build-uri scheme #:path path))
+                             ((uri? path) path)
+                             (else (throw 'artanis-err 500 redirect-to
+                                          "Invalid path ~a, should be string or uri" path))))
                (content-type . (text/html)))))
 
 (define (reject-method method)
@@ -189,7 +239,8 @@
   (call-with-values
       (lambda ()
         (cond
-         ((is-guile-compatible-server-core? (get-conf '(server engine)))
+         ((or (not (get-conf '(server sendfile)))
+              (is-guile-compatible-server-core? (get-conf '(server engine))))
           (generate-response-with-file
            filename
            ;; FIXME: For now, guile compatable server-core doesn't provide good method to
@@ -204,18 +255,21 @@
              (make-file-sender
               size
               (lambda ()
-                ;; NOTE: In Linux, non-blocking for regular file (not a socket) is
-                ;;       basically unsupported!!! So we have to find a way to make sure
-                ;;       the regular file reading is non-blocking, or the whole Ragnarok
-                ;;       will be blocked.
-                ;; TODO: use splice to make a real non-blocking version.
-                ;; TODO: support trunked length requesting for continously downloading.
-                (future
-                 (begin
-                   (sendfile out in size)
-                   (force-output out)
-                   (DEBUG "File `~a' sending over!" filename)
-                   (close in))))))))))
+                (catch #t
+                  (lambda ()
+                    ;; NOTE: In Linux, non-blocking for regular file (not a socket) is
+                    ;;       basically unsupported!!! So we have to find a way to make sure
+                    ;;       the regular file reading is non-blocking, or the whole Ragnarok
+                    ;;       will be blocked.
+                    ;; TODO: use splice to make a real non-blocking version.
+                    ;; TODO: support trunked length requesting for continously downloading.
+                    (sendfile out in size)
+                    (force-output out)
+                    ;;(DEBUG "File `~a' sent over!" filename)
+                    (close in))
+                  (lambda e
+                    (close in)
+                    (apply throw e))))))))))
     (lambda (mtime status body mime)
       (cond
        ((= status 200)
@@ -227,6 +281,8 @@
        (else (response-emit body #:status status))))))
 
 ;; When you don't want to use cache, use static-page-emitter.
-(define* (static-page-emitter rc #:key (dir ""))
-  (emit-response-with-file (string-append dir (static-filename (rc-path rc)))
-                           (request-port (rc-req rc))))
+(define* (static-page-emitter rc #:key (dir #f))
+  (let ((filename (if dir
+                      (format #f "~a/~a" dir (rc-path rc))
+                      (static-filename (rc-path rc)))))
+    (emit-response-with-file filename (request-port (rc-req rc)))))
